#include <QtGui>

#include "jtmainwindow.h"
#include "jtsettingsdialog.h"
#include "jtreportdialog.h"
#include "jthelpdialog.h"

const QString JOBTIME      = ":/jobtime.png";
const QString PAUSETIME    = ":/pausetime.png";
const QString OVERTIME     = ":/overtime.png";
const QString SETTINGSFILE = "jobbtime.ini";
const short SAVETIMEOUT    = 5*60; //5 mins
const short TRAYWARNING    = 10;   //10 mins before overtime

/**
 * global variable are only used for settings parameters
 */
bool g_hideOnStart;
QString g_reportPath;
//QTime g_autocloseTime;


/**
 * Constructor
 */
QjtMainWindow::QjtMainWindow() : QMainWindow()
{
	setWindowTitle("Jobbtime");
	setWindowIcon(QIcon(":/jobtime.png"));
	
	//retrieve the settings, among other in the m_xxxTimes arrays
	m_arrivTimes = new QTime[7];
	m_workingTimes = new QTime[7];
	m_pauseTimes = new QTime[7];
	readSettings();

	m_isWorking = true;
	m_saveTimeout = 1;
	m_trayIconPng = JOBTIME;
	
	/*****************************************
	 * WIDGETS
	 ****************************************/
	//System tray icon
	m_trayIcon = new QSystemTrayIcon(this);
		m_trayIcon->setIcon(QIcon(m_trayIconPng));
		m_trayIcon->show();
	
	//we initiate the 3 spinboxes TimeEdit with values from default settings
	m_arrivTimeEdit = new QTimeEdit(this);
	m_arrivTimeEdit->setDisplayFormat("hh:mm");
	m_pauseTimeEdit = new QTimeEdit(this);
	m_pauseTimeEdit->setDisplayFormat("hh:mm:ss");
	
	m_deparLineEdit = new QLineEdit(this);
	m_deparLineEdit->setReadOnly(true);
	m_deparLineEdit->setMaximumWidth( 50 );

	//circular progress meter
	m_progress = new QjtProgressMeter(this);
	
	this->setupMenus();
	this->setupWidgets();

	
	/**********************************************
	 * Timer initialization
	 *********************************************/
	//retrieve day of the week from the system
	m_weekday = QDate::currentDate().dayOfWeek();
	m_weekday -= 1; //because we fill all the arrays from 0 and dayOfWeek starts at 1
	
	//calculate departTime so we can initialize the timer and m_deparLineEdit
	QTime deparTime = computeDepartTime( m_arrivTimes[m_weekday], m_workingTimes[m_weekday], m_pauseTimes[m_weekday] );
	
	//timer to keep track of all those working hours.
	m_timer = new QjtTimer( m_arrivTimes[m_weekday], deparTime, this);
	
	/********************************************
	 * connections
	 *******************************************/
	this->setupConnections();
	m_deparLineEdit->setText( deparTime.toString("hh:mm") );//m_deparLineEdit is set first since it can be modified when setting up the timeEdits
	this->setDefaultTimeEdits();

}

/**
 * Implement a destructor since we use arrays
 */
QjtMainWindow::~QjtMainWindow()
{
	delete[] m_workingTimes;
	delete[] m_arrivTimes;
	delete[] m_pauseTimes;
}



/*******************************************
 * private functions
 *******************************************/
/**
 * Classical readSettings function
 * Uses the special array serialization of settings provided by Qt
 */
void QjtMainWindow::readSettings()
{
	QSettings settings( SETTINGSFILE, QSettings::IniFormat);
	QPoint pos = settings.value("pos", QPoint(0,0)).toPoint();
	g_hideOnStart = settings.value("hideonstart", false).toBool();
	//g_autocloseTime = settings.value("autoclosetime", QTime( 2, 0)).toTime();
	g_reportPath = settings.value("reportpath", QString("report.csv")).toString();

	settings.beginReadArray("arrivTimes");
	for (int i = 0; i < 7; ++i, settings.setArrayIndex(i)) 
	{
		m_arrivTimes[i] = settings.value("time").toTime();
		if (! m_arrivTimes[i].isValid() )
			m_arrivTimes[i] = QTime::fromString("09:25", "hh:mm");
	}
	settings.endArray();
	
	settings.beginReadArray("workingTimes");
	for (int i = 0; i < 7; ++i, settings.setArrayIndex(i)) 
	{
		m_workingTimes[i] = settings.value("time").toTime();
		if (! m_workingTimes[i].isValid() )
			m_workingTimes[i] = QTime::fromString("08:00", "hh:mm");
	}
	settings.endArray();
	
	settings.beginReadArray("pauseTimes");
	for (int i = 0; i < 7; ++i, settings.setArrayIndex(i)) 
	{
		m_pauseTimes[i] = settings.value("time").toTime();
		if (! m_pauseTimes[i].isValid() )
			m_pauseTimes[i] = QTime::fromString("00:30", "hh:mm");
	}
	settings.endArray();

	move(pos);
}


/**
 * Classical writeSettings function
 * Uses the special array serialization of settings provided by Qt
 */
 void QjtMainWindow::writeSettings()
{
	QSettings settings( SETTINGSFILE, QSettings::IniFormat );
	//settings.setValue( "translation", m_translation );
	settings.setValue( "pos", pos() );
	settings.setValue("hideonstart", g_hideOnStart );
	//settings.setValue("autoclosetime", g_autocloseTime );
	settings.setValue("reportpath", g_reportPath );
	
	settings.beginReadArray("arrivTimes");
	for (int i = 0; i < 7; ++i, settings.setArrayIndex(i))
		settings.setValue("time", m_arrivTimes[i]);
	settings.endArray();
	
	settings.beginReadArray("workingTimes");
	for (int i = 0; i < 7; ++i, settings.setArrayIndex(i)) 
		settings.setValue("time", m_workingTimes[i]);
	settings.endArray();
	
	settings.beginReadArray("pauseTimes");
	for (int i = 0; i < 7; ++i, settings.setArrayIndex(i))
		settings.setValue("time", m_pauseTimes[i]);
	settings.endArray();
}


/**
 * Part of class creation.
 * create the menubar menus and systray icon menu.
 */
void QjtMainWindow::setupMenus()
{
	/**********create Actions*********/
	QAction *settingsAct = new QAction(tr("&Settings"), this);
	connect(settingsAct, SIGNAL( triggered() ), this, SLOT( showSettings() ));
	
	QAction *reportAct = new QAction(tr("&Report"), this);
	connect(reportAct, SIGNAL(triggered()), this, SLOT( showReport() ));
	
	QAction *helpAct = new QAction(tr("&Help"), this);
	connect(helpAct, SIGNAL(triggered()), this, SLOT( showHelp() ));

	QAction *aboutAct = new QAction(tr("&About Jobbtime"), this);
	connect(aboutAct, SIGNAL(triggered()), this, SLOT(about()));
	
	QAction *aboutQtAct = new QAction(tr("About &Qt"), this);
	connect(aboutQtAct, SIGNAL(triggered()), qApp, SLOT( aboutQt() ));
	
	/***** Tray Icon actions********/
	QAction *pauseAct = new QAction(tr("&Pause"),this);
	connect(pauseAct,SIGNAL( triggered() ), this, SLOT( togglePause() ));

	QAction *restoreAct = new QAction(tr("&Restore"),this);
	connect(restoreAct,SIGNAL(triggered()), this, SLOT(show() ));
	
	QAction *quitAct = new QAction(tr("&Quit"), this);
	connect(quitAct,SIGNAL(triggered()), this, SLOT(close() ));

	/*************Create menus*******/
	menuBar()->addAction(settingsAct);
	menuBar()->addAction(reportAct);
	menuBar()->addAction(helpAct);

	QMenu *aboutMenu = menuBar()->addMenu(tr("&About"));
	aboutMenu->addAction(aboutAct);
	aboutMenu->addAction(aboutQtAct);

	//Construct the trayIconMenu and associate it to m_trayIcon
	QMenu *trayMenu = new QMenu();
	trayMenu->addAction(pauseAct);
	trayMenu->addAction(reportAct);
	trayMenu->addAction(restoreAct);
	trayMenu->addAction(quitAct);
	m_trayIcon->setContextMenu(trayMenu);
}


/**
 * Part of class creation.
 * Layout the widgets.
 */
void QjtMainWindow::setupWidgets()
{
	//Central widget
	QWidget *centralWidget = new QWidget(this);

	QVBoxLayout *mainLayout = new QVBoxLayout();
		QHBoxLayout *hLayout = new QHBoxLayout();
		hLayout->addWidget(m_arrivTimeEdit);
		hLayout->addWidget(m_pauseTimeEdit);
		hLayout->addWidget(m_deparLineEdit);
	mainLayout->addLayout(hLayout);
	mainLayout->addWidget(m_progress);
	
	centralWidget->setLayout(mainLayout);
	setCentralWidget(centralWidget);
}


/**
 * Setup the connections between the different members of QjtMainWindow
 */
void QjtMainWindow::setupConnections()
{
	//the most important connection, m_timer->timeout() calls this->timeout()
	connect(m_timer, SIGNAL( updateProgressMeter(double, QTime) ), this, 
			SLOT( timeout( double, QTime ) ));
 
	//if the user clicks on the progressMeter, the pause mode is activated/stopped.
	connect(m_progress, SIGNAL( clicked() ), this, SLOT( togglePause() ));
	
	//action on the systray icon
	connect(m_trayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)), this,
		 	SLOT(iconActivated(QSystemTrayIcon::ActivationReason)));

	//A change to m_arrivTimeEdit implies to re-compute the departure time in a SLOT.
	connect(m_arrivTimeEdit, SIGNAL( timeChanged(const QTime &) ), this, 
				SLOT( arrivTimeChanged(const QTime &) ));
	//A change to m_pauseTimeEdit implies to re-compute the departure time in a SLOT.
	connect(m_pauseTimeEdit, SIGNAL( timeChanged(const QTime &) ), this, 
				SLOT( pauseTimeChanged(const QTime &) ));

}

/**
 * initialize the arrival timeEdit and pause timeEdit with default values
 */
void QjtMainWindow::setDefaultTimeEdits()
{
	m_arrivTimeEdit->setTime( m_arrivTimes[m_weekday] );//call a slot
	m_pauseTimeEdit->setTime( m_pauseTimes[m_weekday] );//call a slot
}


/**
 * calculate the departure Time given arrival time, working and pause time.
 */
QTime QjtMainWindow::computeDepartTime( QTime arrivTime, QTime workingTime, QTime pauseTime )
{
	long workingTimeInSecs = QTime().secsTo( workingTime );
	long pauseTimeInSecs = QTime().secsTo( pauseTime );
	QTime departTime = arrivTime.addSecs( pauseTimeInSecs ).addSecs( workingTimeInSecs );
	
	return departTime;
}


/**
 * Write time data to report.csv, using a model for flexibility / possible future features.
 */
void QjtMainWindow::saveData()
{
	QjtStandardItemModel *jtModel = new QjtStandardItemModel(2, 6, this);
	jtModel->writeReportFile( g_reportPath, m_arrivTimeEdit->time(), m_pauseTimeEdit->time(), m_workingTimes );
}



/********************************************
 * private SLOTS
 ********************************************/
 /**
  * called on m_timer timeout by the timer signal updateMainWindow().
  */
void QjtMainWindow::timeout(double elapsedTimeFraction, QTime remainingTime)
{
	//pop-up a message TRAYWARNING minutes before overtime
	long remainingTimeInSecs = QTime(0,0,0,0).secsTo( remainingTime );
	if ( (remainingTimeInSecs < 60.2*TRAYWARNING ) && (remainingTimeInSecs >= 60*TRAYWARNING) )
		m_trayIcon->showMessage(tr("Jobbtime"), tr("10 mins"), QSystemTrayIcon::Information, 5000);


	//systray ICON
	if ( !m_isWorking )//pause handling
	{
		if ( m_trayIconPng == JOBTIME || m_trayIconPng == OVERTIME )
		{//we do this test redundancy to set new icon only once, not every half second!
			m_trayIconPng = PAUSETIME;
			m_trayIcon->setIcon(QIcon(m_trayIconPng));
		}
		//the pause mode is activated, the pause time increases at each timeout.
		QTime pauseTime = m_pauseTimeEdit->time();
		long intervalInSecs = m_timer->interval() / 1000;
		m_pauseTimeEdit->setTime( pauseTime.addSecs(intervalInSecs) );
	}
	else
	{
		if ( m_trayIconPng == PAUSETIME )
		{//we do this test redundancy to set new icon only once, not every half second!
			m_trayIconPng = (remainingTimeInSecs > 0) ? JOBTIME: OVERTIME;
			m_trayIcon->setIcon(QIcon(m_trayIconPng));
		}
		else if ( elapsedTimeFraction > 1 && m_trayIconPng == JOBTIME )
		{
			m_trayIconPng = OVERTIME;
			m_trayIcon->setIcon(QIcon(m_trayIconPng));
		}
		else if ( elapsedTimeFraction < 1 && m_trayIconPng == OVERTIME )
		{
			m_trayIconPng = JOBTIME;
			m_trayIcon->setIcon(QIcon(m_trayIconPng));
		}
	}


	//update the progress widget
	m_progress->timeout( elapsedTimeFraction, remainingTime, m_isWorking );

	//save to report every SAVETIMEOUT seconds.
	--m_saveTimeout;
	if ( m_saveTimeout == 0 )
	{
		m_saveTimeout = SAVETIMEOUT;
		this->saveData();
	}
}


/**
 * transforms an arrival time change from the user into a departure time change
 * in order to display the new departure time.
 */
void QjtMainWindow::arrivTimeChanged(const QTime &arrivTime)
{
	QTime currentTime = QTime::currentTime();
	if ( currentTime < arrivTime )
	{
	/* 
	 * the user is setting an arrival time 'in the future'.
	 */
		/*
		 * Display a warning box, except if the 'future' time is the default arrival
		 * time, which would probably mean that the user is just arriving earlier than 
		 * default time at work for this particular day.
		 */
		if ( arrivTime != m_arrivTimes[m_weekday] )
			QMessageBox::warning(this, tr("You are already there!"),
					QString(tr("Arrival time is set to 'now' for today (")).append( currentTime.toString("hh:mm") ).append(") instead of ").append(
					arrivTime.toString("hh:mm")));

		m_arrivTimeEdit->setTime( currentTime );//call back this slot!
	}
	else
	{
		QTime pauseTime = m_pauseTimeEdit->time();
		QTime deparTime = computeDepartTime( arrivTime, m_workingTimes[m_weekday], pauseTime );
		
		m_timer->arrivTimeChanged( arrivTime );
		m_deparLineEdit->setText( deparTime.toString("hh:mm") );
		m_timer->deparTimeChanged( deparTime );
	}
}

	
/**
 * transforms a pause time change from the user into a departure time change
 * in order to display the new departure time.
 * Send a signal to notify the timer of the departure time change.
 */
void QjtMainWindow::pauseTimeChanged(const QTime &pauseTime)
{
	QTime arrivTime = m_arrivTimeEdit->time();
	QTime deparTime = QTime::fromString( m_deparLineEdit->text(), "hh:mm" );
	
	deparTime = computeDepartTime( arrivTime, m_workingTimes[m_weekday], pauseTime );
	m_deparLineEdit->setText( deparTime.toString("hh:mm") );
	m_timer->deparTimeChanged( deparTime );
}


/**
 * The user takes a pause manually.
 * In pause mode, the timer is not stopped. Instead the pause time is increased in realtime,
 * which modifies the departure time in realtime since the two are connected.
 * We just need to update m_pauseTimeEdit ( which itself updates m_deparLineEdit ) in this->timeout().
 * m_isWorking is used in this->timeout().
 */
void QjtMainWindow::togglePause()
{
	m_isWorking = !m_isWorking;
}


/** 
 * called when the system tray icon is clicked on.
 */
void QjtMainWindow::iconActivated(QSystemTrayIcon::ActivationReason reason)
{
	switch (reason) 
	{
		case QSystemTrayIcon::Trigger:
		case QSystemTrayIcon::MiddleClick:
		case QSystemTrayIcon::DoubleClick:
			show();
			activateWindow();
			break;
		default:
			break;
	}
}


/**
 * Show the settings dialog box on action request
 */
void QjtMainWindow::showSettings()
{
	QjtSettingsDialog *settingsDialog = new QjtSettingsDialog( m_arrivTimes, m_pauseTimes, m_workingTimes, this );
	connect( settingsDialog, 
	SIGNAL( save(QTimeEdit *, QTimeEdit *, QTimeEdit *) ), 
	SLOT( saveSettings(QTimeEdit *, QTimeEdit *, QTimeEdit *) ));
	settingsDialog->exec();
}
/** 
 * Set default time according to the result of showSettings().
 */
void QjtMainWindow::saveSettings(QTimeEdit * arrivTimeEdit, QTimeEdit * pauseTimeEdit, QTimeEdit * workingTimeEdit)
{
	for (short i=0; i<7; ++i)
	{
		m_arrivTimes[i] = arrivTimeEdit[i].time();
		m_workingTimes[i] = workingTimeEdit[i].time();
		m_pauseTimes[i] = pauseTimeEdit[i].time();
	}

	this->setDefaultTimeEdits();
}


/**
 * show the report dialog box on action request
 */
void QjtMainWindow::showReport()
{
	this->saveData();//in order to display an up-to-date information.
	QjtReportDialog *reportDialog = new QjtReportDialog( g_reportPath, this );
	reportDialog->exec();
}


void QjtMainWindow::showHelp()
{
	QjtHelpDialog *helpDialog = new QjtHelpDialog( this );
	helpDialog->show();
}


/**
 * About jobbtime dialog.
 */
void QjtMainWindow::about()
{
	QMessageBox* aboutBox = new QMessageBox(this);
	aboutBox->setWindowTitle( "About Jobbtime" );
	aboutBox->setIconPixmap( QPixmap(":/jobtime.png") );
	aboutBox->setText( tr(
			"<p>Jobbtime is an open source application under GPL2 license. Please refer to "
			"<a href=\"http://www.gnu.org/licenses/\">http://www.gnu.org/licenses/</a> for the license details."
			"<p>Author : Ludovic Angot"
			"<br/>Website: <a href=\"http://www.vokabb.org\">jobbtime.vokabb.org</a></p>"
			"<p>Thanks to Przemyslaw Pawelczyk and Christoph Maciejewski for their inspiring softwares.</p>"
			     ));
	aboutBox->setTextFormat(Qt::RichText);
	aboutBox->exec();
}



/********************************************
 * protected functions
 ********************************************/
/**
 * Overload closeEvent function
 */
void QjtMainWindow::closeEvent(QCloseEvent *event)
{
	this->saveData();//working time data
	this->writeSettings();//application data
	event->accept();
	QCoreApplication::quit();
}

/**
 * We overload changeEvent to reduce the window
 * to systray instead of the taskbar when 'minimizing'.
 * Side effect : when 'maximizing' the window
 * is reduced to systray too.
 */
void QjtMainWindow::changeEvent ( QEvent *event ) 
{
	if ( event->type() == QEvent::WindowStateChange )
	{
		if ( isVisible() )
		{
			event->ignore();
			setWindowState( Qt::WindowNoState );
			hide();
		}
	}
	else
		event->accept();
}

