#include <QtGui>
#include <math.h>

#include "jtprogressmeter.h"


QjtProgressMeter::QjtProgressMeter(QWidget* parent) : QPushButton(parent)
{
	m_elapsedTimeFraction = 0;
	m_remainingTime = QTime(0,0,0,0);

	m_backgroundWorkTime = QColor(255, 255, 150);//yellow
	m_workedTime         = QColor(150, 255, 150);//green
	m_backgroundOverTime = QColor(150, 255, 150);//green
	m_overtimeTime       = Qt::red;

	setMinimumSize(minimumSizeHint());
	setToolTip(tr("click to pause"));
}


/*************************************************
 * public functions
 ************************************************/
/**
 * overload the minimumSizeHint() function
 */
QSize QjtProgressMeter::minimumSizeHint() const
{
	return QSize(150,150);
}


/**
 * set the elapsedTimeFraction of working time
 * set the time visible in the middle of the circle
 */
void QjtProgressMeter::timeout( double elapsedTimeFraction, QTime remainingTime, bool isWorking )
{
	m_elapsedTimeFraction = elapsedTimeFraction;
	m_remainingTime = remainingTime;
	m_isWorking = isWorking;
	repaint();
}


/*****************************************************
 * protected functions
 ****************************************************/
/**
 * overload QPaintEvent to use or own painter and drawer
 */
void QjtProgressMeter::paintEvent(QPaintEvent* /* event */)
{
	QPainter painter(this);
	painter.setRenderHint(QPainter::Antialiasing);
	painter.setWindow(- width() / 2, - height() / 2, width(), height());
	//side defined as below allows the disk to stay inside the buttonBorders of the widget
	double side = qMin(width(), height()) / 2.0;

	double buttonBorder = 10;//10px
	drawMeter(&painter, side, buttonBorder);
}

/*************************************************
 * private functions
 ************************************************/
/**
 * draw the widget.
 */
void QjtProgressMeter::drawMeter(QPainter *painter, double radius, double buttonBorder)
{
	QPointF outter     = QPointF(radius, radius);
	QRectF  outterRect = QRectF(-outter, outter); 
	QPointF inner      = outter - QPointF(buttonBorder, buttonBorder);
	QRectF  innerRect  = QRectF(-inner, inner);

	/*draw the border of the button*/
	painter->setPen(Qt::NoPen);
	QRadialGradient rGradient;
	rGradient = QRadialGradient(outter * -0.5, radius * 1.5);
	rGradient.setColorAt(0.0, palette().light().color());
	rGradient.setColorAt(1.0, palette().dark().color());
	painter->setBrush(rGradient);
	painter->drawEllipse(outterRect);
	
	if (buttonBorder >= radius)
		return;


	/*
	 * if the fraction of elapsedTime becomes higher than one, we are overtime.
	 * In this case, we restart painting from the beginning but with another color
	 */

	/*draw the button itself*/
	if ( m_elapsedTimeFraction <= 1 )
	{
		if ( m_isWorking )
			painter->setBrush( QBrush(m_backgroundWorkTime) );
		else
			painter->setBrush( QBrush(palette().dark().color()) );
	}
	else
		painter->setBrush( QBrush(m_backgroundOverTime) );

	painter->drawEllipse(innerRect);


	/*draw the pie of completed time over the button*/
	if ( m_elapsedTimeFraction <= 1 )
	{
		if ( m_isWorking )
			painter->setBrush( QBrush(m_workedTime) );
		else
			painter->setBrush( QBrush(palette().light().color()) );
	}
	else
		painter->setBrush( QBrush(m_overtimeTime) );

	//the modulo 5760 means that we restart the pie from 0 when on overtime (with other colors)
	painter->drawPie(innerRect, (5760/4), (int)(m_elapsedTimeFraction * -5760) % 5760 );

	painter->setPen(palette().text().color());
	painter->drawText(innerRect, Qt::AlignCenter | Qt::TextDontClip, m_remainingTime.toString("hh::mm:ss"));

}

