#include <QtGui>
#include <cmath>

#include "jtstandarditemmodel.h"

/******************************************************
 * QjtStandardItemModel implementation
 ******************************************************/

QjtStandardItemModel::QjtStandardItemModel(int rows, int columns, QObject * parent) : QStandardItemModel(rows, columns, parent)
{	
	m_rowCount = 0;//number of rows actually filled in the models

	setHeaderData(0, Qt::Horizontal, "date");
	setHeaderData(1, Qt::Horizontal, "arrival");
	setHeaderData(2, Qt::Horizontal, "pause");
	setHeaderData(3, Qt::Horizontal, "departure");
	setHeaderData(4, Qt::Horizontal, "overtime in hours");
	setHeaderData(5, Qt::Horizontal, "total");
}



/**********************************
 * private functions
 **********************************/
/**
 * Tell a view which is the last valid index to allow efficient display
 * The report file data must have been before.
 * @arg visibleRows: the number of visible rows in the calling view.
 */
const QModelIndex QjtStandardItemModel::getScrollIndex( short visibleRows )
{
	if ( m_rowCount < visibleRows && index(0,0).isValid() )
		return index( 0, 0);
	else if ( index(m_rowCount-1, 0).isValid() )
		return index(m_rowCount - 1- visibleRows / 2, 0);
	//else
	return QModelIndex();
}


/**
 * Return a human readable QString out of a double.
 * We don't use the QTime class for overtime since the display is limited to 23:59:59.
 */
const QString QjtStandardItemModel::getOvertimeString( double overtime )
{
/*
 * This commented implementation doesn't work on all platforms
 *	double hours;
 *	double minutes = floor( modf( overtime, &hours ) * 60 );
 *
 *	QString s_hours   = QString().setNum( (int)hours );
 *	QString s_minutes = QString().setNum( abs( (int)minutes ) );
 *
 */
	double hours, minutes;

	hours = overtime > 0 ? floor(overtime) : ceil(overtime);
	QString s_hours   = QString().setNum( hours, 'f', 0 );

	if ( hours == 0 )
        {//we want a minus sign in front of the hours, for example -0h 32mins
                if ( overtime < 0 )
                        s_hours.prepend("-");
                else
                        s_hours.prepend(" ");
        }


 	hours = fabs( hours );
        minutes = overtime > 0 ? (overtime - hours) : (overtime + hours);
        minutes = fabs( floor(minutes*60) );
	QString s_minutes = QString().setNum( minutes, 'f', 0 ); 

	
        return s_hours.append("h ").append(s_minutes).append("mins");
}


/**
 * @arg fileName : the name of the file to open (example.csv,...)
 *return the total cumulative overtime as a QTime, which is very convenient 
 * for a nice display, or an invalid QTime if there is any file error.
 */
bool QjtStandardItemModel::readReportFile(QString fileName)
{
	QFile reportFile(fileName);
	if ( reportFile.exists() )
	{
		if (reportFile.open(QIODevice::ReadOnly | QFile::Text))
		{
			QTextStream stream(&reportFile);
			QString line;
			QStringList tokens;

			//we append the data of this file to the current model
			while ( line = stream.readLine(), !line.isEmpty() )
			{
				tokens = line.split(",");

				for ( short i=0; i<6; ++i )
					setData(index(m_rowCount, i), tokens.value(i));

				if ( tokens.value(5).contains("-") )
					setData(index(m_rowCount, 5),
							QColor( 255, 240, 240 ), 
							Qt::BackgroundRole);
				else	
					setData(index(m_rowCount, 5), 
							QColor( 240, 255, 240 ),
							Qt::BackgroundRole);

				++m_rowCount;
				insertRow( m_rowCount );//insert a row a the very end of the model to compensate new data.
			}

			reportFile.close();
		} 
		else
			return false;
	} 
	else
		return false;

	return true;
}


/**
 * Write data to the report file
 * @arg fileName   : the name of the file to open (example.csv,...)
 * @arg arrivTime  : the arrival time for today
 * @arg pauseTime  : the pause time for today
 * @arg workingTime: an array containing the default working time for the 7 days of the week.
 */
bool QjtStandardItemModel::writeReportFile(QString filename, QTime arrivTime, QTime pauseTime, QTime *workingTime)
{
	QFile reportFile(filename);
	QString line;

	QDate currentDate = QDate::currentDate();
	QTime currentTime = QTime::currentTime();

	int overtimeInSecs = 0;
	double overtimeInHours = 0;
	double cumulativeOvertimeInHours = 0;
	short i;//used in loops

	if ( !reportFile.exists() )
	{//create it
		reportFile.open(QFile::WriteOnly | QFile::Text);
		reportFile.close();
	}

	/****************************** get file data as a model **********************************/
	if ( reportFile.open(QFile::ReadOnly | QFile::Text) )
	{
		QTextStream stream(&reportFile);
		QTime aTime, pTime, dTime;
		QDate rowDate;
		int rowDay;

		/*
		 * First we search if data have already been written today.
		 * And we re-calculate the previous cumulative overtime.
		 * Re-calculation allows external modifications of
		 * the csv file.
		 */
		while ( line = stream.readLine(), !line.isEmpty() )
		{
			QStringList tokens = line.split(",");

			//Test if we have already written data for 'today'
			if ( tokens.value(0).localeAwareCompare( currentDate.toString() ) == 0 )
				break;
			
			//else we retrieve data
			for ( i=0; i < 4; ++i )
				setData(index(m_rowCount, i), tokens.value(i));

			//we retrieve the day of the week {1,...,7} for this row
			rowDate = data( index(m_rowCount, 0) ).toDate();
			rowDay = rowDate.dayOfWeek();

			aTime = data(index(m_rowCount, 1)).toTime();//arrival time for this row
			pTime = data(index(m_rowCount, 2)).toTime();//pause time for this row
			dTime = data(index(m_rowCount, 3)).toTime();//departure time for this row

			//overtime == worked time - time-to-be-worked - pause
			overtimeInSecs = aTime.secsTo( dTime ) 
						- QTime(0,0,0,0).secsTo( workingTime[rowDay] ) 
						- QTime(0,0,0,0).secsTo( pTime );
			overtimeInHours = ((double)overtimeInSecs) / 3600;
			setData(index(m_rowCount, 4), getOvertimeString( overtimeInHours ));

			cumulativeOvertimeInHours += overtimeInHours;
			setData(index(m_rowCount, 5), getOvertimeString( cumulativeOvertimeInHours ));

			++m_rowCount;
			insertRow(m_rowCount );//insert a row a the very end of the model to compensate new data.
		}

		reportFile.close();
	}
	else
		return false;


	/*********** Write the data for 'today'. At this point m_rowCount points to 'today' row. ************/
	setData(index(m_rowCount, 0), currentDate.toString());
	setData(index(m_rowCount, 1), arrivTime);
	setData(index(m_rowCount, 2), pauseTime);
	setData(index(m_rowCount, 3), currentTime);

	int rowDay = currentDate.dayOfWeek();

	overtimeInSecs = arrivTime.secsTo( currentTime ) 
				- QTime(0,0,0,0).secsTo( workingTime[rowDay] ) 
				- QTime(0,0,0,0).secsTo( pauseTime );

	overtimeInHours = ((double)overtimeInSecs) / 3600;
	setData(index(m_rowCount, 4), getOvertimeString( overtimeInHours ));

	cumulativeOvertimeInHours += overtimeInHours;
	setData(index(m_rowCount, 5), getOvertimeString( cumulativeOvertimeInHours ));

	++m_rowCount;


	/********************** Write the modified model back to the report file. *************************/
	if (reportFile.open(QFile::WriteOnly | QFile::Text))
	{
		QTextStream stream(&reportFile);

		for (int row=0; row < m_rowCount; ++row)
		{
			for ( i = 0; i < 6 ; ++i )
				stream << data(index(row, i)).toString() << ",";
			stream << "\n";
		}
		reportFile.close();
	}
	else
		return false;


	return true;
/*
	if (reportFile.open(QFile::ReadWrite | QFile::Text))
	{
		QString currentDate = QDate::currentDate().toString();
		QTextStream stream(&reportFile);
		QString line;
		qint64 linePosition= Q_INT64_C(0);

		while ( linePosition = stream.pos(), line = stream.readLine(), !line.isEmpty() )
		{
			QStringList tokens = line.split(",");
			
			if ( (tokens.value(0)).localeAwareCompare( currentDate ) == 0 )
			{//so we erase previous data!
				stream.seek(linePosition);
				for ( short i=0; i < line.length(); ++i)
					stream << " ";
				break;
			}
		}

		stream.seek(linePosition);
		stream << currentDate << ", " << "\n";

		reportFile.close();
		return true;
	}
*/
}


