#include <QtGui>

#include "jttimer.h"

QjtTimer::QjtTimer( QTime arrivTime, QTime deparTime, QObject *parent ) : QTimer(parent)
{
	m_arrivTime = arrivTime;
	m_deparTime = deparTime;
	m_elapsedFromArrivTimeInSecs = 0;//the time elapsed between theoric arrival time and timer start
	m_elapsedFromStartTimeInSecs = 0;//counted since timer start() to now
	/*
	 * For example the default arrival time was set to 8.15 and
	 * the person actually arrived at 8.23. Time is now 9.13.
	 * m_elapsedFromArrivTimeInSecs = 8mins * 60secs from 8.15 to 8.23
	 * m_elapsedFromStartTimeInSecs = 50mins * 60secs from 8.23 to 9.13
	 * 
	 * m_elapsedFromArrivTimeInSecs is NOT overriden when starting jobtime. So m_elapsedFromArrivTimeInSecs
	 * is set to 0 even if the person arrives at 8.23 instead of 8.15. we do so because there is a possible
	 * delay between arrival time and starting the computer/jobtime!
	 * However m_elapsedFromArrivTimeInSecs will be used in this->arrivTimeChanged() if the user
	 * overrides the arrival time manually. For example if he changes his arrival time from 8.15 to 8.23.
	 */
	connect( this, SIGNAL( timeout() ), this, SLOT( update() ));

	start(1000);
}

/********************************
 * private SLOTS
 *******************************/

/**
 * keep track of elapsedTime and perform calculations usefull for updating MainWindow
 */
void QjtTimer::update()
{
	//update the elapsed time since timer start in seconds
	m_elapsedFromStartTimeInSecs += (double)interval() / 1000;
	double elapsedTimeInSecs = m_elapsedFromArrivTimeInSecs + m_elapsedFromStartTimeInSecs;

	//calculate the total time to-stay-at-work (work+pause) in seconds;
	double totalTimeInSecs = m_arrivTime.secsTo( m_deparTime );

	//calculate the fraction that m_elapsedTime represents compared to totalTime (max = 1 so)
	double elapsedTimeFraction = elapsedTimeInSecs / totalTimeInSecs ;


	//calculate the remaining time as a QTime for user display in the middle of the progress meter
	QTime remainingTime;
	if ( elapsedTimeFraction <= 1 )
		remainingTime =  QTime(0, 0, 0, 0).addSecs( (int) (totalTimeInSecs - elapsedTimeInSecs) );
	else
		remainingTime =  QTime(0, 0, 0, 0).addSecs( (int) (elapsedTimeInSecs - totalTimeInSecs) );


	//if the fraction of elapsedTime becomes higher than one, we are overtime 
	emit updateProgressMeter( elapsedTimeFraction, remainingTime );
}

/**
 * the timer needs the arrrival and departure Times to compute 
 * the overtime.
 * So timeEdit changes in QjtMainWindow are directly linked here.
 */
void QjtTimer::arrivTimeChanged(const QTime & time)
{
	m_arrivTime = time;
	//check how many seconds we have already worked today considering the new m_arrivTime
	m_elapsedFromArrivTimeInSecs = m_arrivTime.secsTo( QTime::currentTime() );
	/*
	 * restart the timer because the 'elapsed time from arrival' is recomputed as the difference
	 * between the new m_arrivTime and currentTime.
	 * For example someone has been working for 50mins, the default arrival time was set to 8.15 and
	 * the person actually arrived at 8.23. Time is now 9.13.
	 * m_elapsedFromArrivTimeInSecs = 8mins * 60secs
	 * m_elapsedFromStartTimeInSecs = 50mins * 60secs
	 * The user changes its arrival time to 9.00 (he was at his workplace earlier but doing personal
	 * stuff :). The time counted by the timer since 8.23 has no more sense. On the other hand it is
	 * very meaningfull to set m_elapsedFromArrivTimeInSecs to the difference between 9.00 and 9.13.
	 * Then we restart the timer from 9.23.
	 */
	stop(); m_elapsedFromStartTimeInSecs = 0; start(1000);
}

void QjtTimer::deparTimeChanged(const QTime & time)
{
	m_deparTime = time;
}

